<?php

/**
 * Database access controller class
 *
 * @package objSQL
 * @version 3.2.0
 * @author MT Jordan <mtjo62@gmail.com>
 * @copyright 2013
 * @license zlib/libpng
 * @link http://objsql.sourceforge.net 
 */

class objSQL extends obj_access
{
    /**********************************************
     * Internal private variables
     *********************************************/

    /**
     * Database connection object
     *
     * @access public
     * @var    mixed
     */
    public $obj_connection = false;
    
    /**
     * Database type array contains all supported database drivers
     *
     * @access private
     * @var    array
     */
    private $obj_db_array = array( 'cubrid',
	                               'mysql',
                                   'pgsql',
                                   'sqlite3',
                                   'sqlsrv',
								   'pdo:cubrid',
								   'pdo:mysql',
								   'pdo:pgsql',
								   'pdo:sqlite3',
								   'pdo:sqlsrv' );

    /**
     * Database driver type as set in datasource argument.
     *
     * @access private
     * @var    str
     */
    private $obj_db_driver;  
	
    /**
     * Database type
     *
     * @access private
     * @var    str
     */
    private $obj_db_type;  
	
    /**
     * Database connection class instance
     *
     * @access private
     * @var    mixed
     */
    private $obj_instance;
    
    /**
     * Database prepared statement string
     *
     * @access private
     * @var    str
     */
    private $obj_prepare;
    
    /**
     * Database query string
     *
     * @access private
     * @var    str
     */
    private $obj_statement;
    
    /**
     * Database transaction string
     *
     * @access private
     * @var    str
     */
    private $obj_transaction;
    
    /**
     * objSQL version
     *
     * @access private
     * @var    str
     */
    private $obj_version = '3.2.0';
    
    /**********************************************
     * Public class methods
     *********************************************/

    /**
     * Constructor
     *
     * @access public
     * @param  mixed $datasource
     */
    public function __construct( $datasource )
    {
        $this->obj_helper_datasource( $datasource );
    }
    
    /**
     * Closes current database connection
     *
     * @access public
     * @return bool
     */
    public function obj_close()
    {
        return $this->obj_instance->obj_db_close();
    }
    
	/**
     * Executes delete query and returns resultset object/resource
     *
     * @access public
     * @param str $table
     * @param str $where
     * @return mixed
     */
    public function obj_delete( $table, $where=false )
    {
        $query_stmt = new $this->obj_statement( $this->obj_helper_delete( $table, $where ), $this, $this->obj_db_type );
        
        return $query_stmt->obj_query_execute();
    }
	
	/**
     * Database driver type
     *
     * @access public
     * @return str
     */
    public function obj_driver()
    {
        return $this->obj_db_driver;
    }
    
    /**
     * Returns error flag for current connection instance - true/false
     *
     * @access public
     * @return bool
     */
    public function obj_error()
    {
        return $this->obj_instance->obj_db_error();
    }
    
    /**
     * Returns error message for current connection instance
     *
     * @access public
     * @return str
     */
    public function obj_error_message()
    {
        return $this->obj_instance->obj_db_message();
    }
    
    /**
     * Returns escaped string data for database insertion
     *
     * @access public
     * @param  str $data
     * @return str
     */
    public function obj_escape( $data )
    {
        return $this->obj_instance->obj_db_escape_data( $data );
    }
    
    /**
     * Returns associative array with general database and script information
     *
     * @access public
     * @return array
     */
    public function obj_info()
    {
        return array( 'OBJSQL_VERSION'   => $this->obj_version,
                      'DATABASE_NAME'    => $this->obj_instance->obj_db_info()[2],
                      'DATABASE_TYPE'    => $this->obj_db_driver,
                      'DATABASE_VERSION' => $this->obj_instance->obj_db_info()[0],
                      'DATABASE_CHARSET' => $this->obj_instance->obj_db_info()[1],
                      'PHP_VERSION'      => phpversion() );
    }
    
    /**
     * executes insert query  and returns resultset object/resource
     *
     * @access public
     * @param  str   $table
     * @param  array $data_array
     * @return mixed
     */
    public function obj_insert( $table, $data_array )
    {
        //$data_array MUST be a key value pair array - value can be a one dim array or comma delimited string
        $query_stmt = new $this->obj_statement( $this->obj_helper_insert( $table, $data_array ), $this, $this->obj_db_type );

        return $query_stmt->obj_query_execute();
    }
    
    /**
     * Executes select paging query and returns resultset object/resource and number of pages via $limit
     *
     * @access public
     * @param  str $table
     * @param  str $cols
     * @param  str $where
     * @param  str $order_by
     * @param  int $limit
     * @param  int $offset
     * @return array
     */
    public function obj_paging( $table, $cols=false, $where=false, $order_by=false, $limit=1, $offset=1 )
    {
        //make sure $limit & $offset are unsigned ints > 0
        $query_offset = ( is_numeric( $offset ) && $offset > 0 ) ? ( int )$offset : 1;
        $query_limit  = ( is_numeric( $limit ) && $limit > 0 ) ? ( int )$limit : 1;
		// 04/21/13 - added where clause to fix last page bug
		$query_where = ( !trim( $where ) ) ? '' : $where; 
		        
        $query_stmt = new $this->obj_statement( $this->obj_helper_paging( $table, $cols, $query_where, $order_by, $query_limit, $query_offset ), $this, $this->obj_db_type );

        return array( $query_stmt->obj_query_execute(), ceil( $this->obj_row_count( $table, $cols, $query_where )/$query_limit ) );
    }
    
    /**
     * Returns prepared statement instance
     *
     * @access public
     * @param  str   $query
     * @param  array $param_vars
     * @return mixed
     */
    public function obj_prepare_statement( $query, $param_vars=false )
    {
        return new $this->obj_prepare( $query, $this, $this->obj_db_type, $param_vars );
    }
    
    /**
     * Executes non-prepared query and returns resultset object/resource
     *
     * @access public
     * @param  str $query
     * @return mixed
     */
    public function obj_query( $query )
    {
        $query_stmt = new $this->obj_statement( $query, $this, $this->obj_db_type );

        return $query_stmt->obj_query_execute();
    }
	
	/**
     * Executes select count query and returns row count
     *
     * @access public
     * @param  str $table
     * @param  str $cols
     * @param  str $where
     * @return int
     */
    public function obj_row_count( $table, $cols=false, $where=false )
    {
        return $this->obj_instance->obj_db_rowcount( $table, $cols, $where );
    }
	
	/**
     * Executes select query and returns resultset object/resource
     *
     * @access public
     * @param  str $table
     * @param  str $cols
     * @param  str $where
     * @param  str $order_by
     * @param  str $sort_order
     * @return mixed
     */
    public function obj_select( $table, $cols=false, $where=false, $order_by=false, $sort_order=false )
    {
        $query_stmt = new $this->obj_statement( $this->obj_helper_select( $table, $cols, $where, $order_by, $sort_order ), $this, $this->obj_db_type );
        
         return $query_stmt->obj_query_execute();
    }
	
	/**
     * Begins transaction     
     * 
     * @access public
     * @return mixed
     */
    public function obj_transaction()
    {
        return new $this->obj_transaction( $this->obj_connection, $this->obj_db_type );
    }

    /**
     * Executes update query and returns resultset object/resource
     *
     * @access public
     * @param  str   $table
     * @param  array $data_array
     * @param  str   $where
     * @return mixed
     */
    public function obj_update( $table, $data_array, $where=false )
    {
        //$data_array MUST be a key value pair array
        $query_stmt = new $this->obj_statement( $this->obj_helper_update( $table, $data_array, $where ), $this, $this->obj_db_type );
        
        return $query_stmt->obj_query_execute();
	}
    
    /**********************************************
     * Private helper methods
     *********************************************/
    
    /**
     * Sets database connection instance/vars and loads database drivers
     *
     * @access private
     * @param  mixed $datasource
     * @return mixed 
     */
    private function obj_helper_datasource( $datasource )
    {
        $obj_datasource = ( is_array( $datasource ) ) ? $datasource : explode( ',', $datasource );
        
        if ( in_array( strtolower( $obj_datasource[0] ), $this->obj_db_array ) )
        {
			$this->obj_db_driver = strtolower( $obj_datasource[0] );
			
			if ( substr_count( $this->obj_db_driver, 'pdo:' ) )
			{
				$this->obj_db_type = str_replace( 'pdo:', '', $this->obj_db_driver );
				$this->obj_statement = "pdo_statement";
				$this->obj_prepare = "pdo_prepare";
				$this->obj_transaction = "pdo_transaction";
				$obj_connection = "pdo_connection";
				
				require_once "drivers/pdo.php";
            }
			else
			{
				$this->obj_db_type = $this->obj_db_driver;
				$this->obj_statement = "{$this->obj_db_type}_statement";
				$this->obj_prepare = "{$this->obj_db_type}_prepare";
				$this->obj_transaction = "{$this->obj_db_type}_transaction";
				$obj_connection = "{$this->obj_db_type}_connection";
				
				require_once "drivers/{$this->obj_db_type}.php";
            }
			
			$this->obj_instance = new $obj_connection( $obj_datasource, $this->obj_db_type );
			
			if ( $this->obj_instance )
                return $this->obj_connection = $this->obj_instance->obj_db_connection();
            else
                return false;
        }
        else
        {
            trigger_error( "Unsupported database type/driver: {$obj_datasource[0]}", E_USER_WARNING ); 
            
            return false;
        }    
    }
    
    /**
     * Returns delete query SQL statement
     *
     * @access private
     * @param str $table
     * @param str $where
     * @return mixed
     */
    private function obj_helper_delete( $table, $where=false )
    {
        $query_where = ( !trim( $where ) ) ? '' : "WHERE $where"; 
        
        return "DELETE FROM $table $query_where";
    }
    
    /**
     * Returns insert query SQL statement
     *
     * @access private
     * @param  str   $table
     * @param  array $data_array
     * @return str
     */
    private function obj_helper_insert( $table, $data_array )
    {
        //$data_array MUST be a key value pair array - value can be a one dim array or comma delimited string
        $array_keys = array_keys( $data_array );
        $array_vals = array_values( $data_array );
        
        //if array_vals[$i] is a string, convert to array
        for ( $i = 0; $i < count( $array_vals ); $i++ )
        {
            if ( !is_array( $array_vals[$i] ) )
            {
                $temp_array = explode( ',', $array_vals[$i] );
                $array_vals[$i] = $temp_array;
            }
        }
        
        for ( $i = 0; $i < count( $array_vals[0] ); $i++ )
        {
            $query_insert[] = '(';
            
            for ( $j = 0; $j < count( $array_vals ); $j++ )
            {
                if ( is_string( $array_vals[$j][$i] ) && !is_numeric( $array_vals[$j][$i] ) )
                    $query_insert[] = "'{$array_vals[$j][$i]}'";  
                else    
                    $query_insert[] = $array_vals[$j][$i];
            }
            
            $query_insert[] = ')';
        }
		
		// 8/4/13 - removed .= operator
        $query_cols = implode( ',', $array_keys ); 
        $query_vals = str_replace( array( ',),(,', ',)', '(,' ), array( '),(', ')', '('), implode( ',', $query_insert ) );
                
        return "INSERT INTO $table ($query_cols) VALUES $query_vals";
    }
    
    /**
     * Returns select paging query SQL statement
     *
     * @access private
     * @param  str $table
     * @param  int $limit 
     * @param  int $offset 
     * @param  str $cols
     * @param  str $where
     * @param  str $order_by
     * @return str
     */
    private function obj_helper_paging( $table, $cols=false, $where=false, $order_by=false, $limit=1, $offset=1 )
    {
        $query_cols  = ( !trim( $cols ) ) ? '*' : $cols;
        $query_where = ( !trim( $where ) ) ? '' : "WHERE $where"; 
        $query_order = ( !trim( $order_by ) ) ? '' : "ORDER BY $order_by";
        $query_offset = ( ( $offset - 1 ) * $limit );
        
        if ( $this->obj_db_type === 'sqlsrv' )
        {
			// check for SQL Server 2012+
			if ( explode( '.',  $this->obj_instance->obj_db_info()[0] ) < 11 )
			{
				$query_sql = "WITH temp_table AS ( 
                              SELECT ROW_NUMBER() OVER ( $query_order ) AS RowNumber, $query_cols 
                              FROM $table $query_where ) 
                              SELECT * 
                              FROM temp_table 
                              WHERE RowNumber > $query_offset AND RowNumber <= ( $limit + $query_offset )";
			}
			else
			{
				// added to support SQL Server 2012 - throws cursor exception using row_number method
				$query_sql = "SELECT $query_cols FROM $table $query_where $query_order
                              OFFSET $query_offset ROWS
                              FETCH NEXT $limit ROWS ONLY";	
			}							  
        }
        else
        {
            if ( $this->obj_db_type === 'pgsql' )
                $limit_str = "LIMIT $limit OFFSET $query_offset";
            else
                $limit_str = "LIMIT $query_offset,$limit"; 
                
            $query_sql = "SELECT $query_cols FROM $table $query_where $query_order $limit_str";
        }
        
        return $query_sql;
    }
	
	/**
     * Returns select query SQL statement
     *
     * @access private
     * @param  str $table
     * @param  str $cols
     * @param  str $where
     * @param  str $order_by
     * @param  str $sort_order
     * @return str
     */
    private function obj_helper_select( $table, $cols=false, $where=false, $order_by=false, $sort_order=false )
    {
        $query_cols  = ( !trim( $cols ) ) ? '*' : $cols;
        $query_where = ( !trim( $where ) ) ? '' : "WHERE $where"; 
        $query_order = ( !trim( $order_by ) ) ? '' : "ORDER BY $order_by";
        $query_sort  = ( !trim( $sort_order ) || strtolower( $sort_order ) != 'desc' || strtolower( $sort_order ) != 'asc' ) ? '' : $sort_order;
        
        return "SELECT $query_cols FROM $table $query_where $query_order $query_sort";
    }
    
    /**
     * Returns update query SQL statement
     *
     * @access private
     * @param  str   $table
     * @param  array $data_array
     * @param  str   $where
     * @return str
     */
    private function obj_helper_update( $table, $data_array, $where=false )
    {
        //$array MUST be a key value pair array
        $query_cols   = array_keys( $data_array );
        $query_vals   = array_values( $data_array );
		$query_update = '';

        for ( $i = 0; $i < count( $data_array ); $i++ )
        {
            if ( is_string( $query_vals[$i] ) && !is_numeric( $query_vals[$i] ) )
                $query_update .= "{$query_cols[$i]}='{$query_vals[$i]}',";
            else
                $query_update .= "{$query_cols[$i]}={$query_vals[$i]},";
        }

        $query_vars = rtrim( $query_update, ',' );
        $query_where  = ( !trim( $where ) ) ? '' : "WHERE $where";
                
        return "UPDATE $table SET $query_vars $query_where";
    }
    
    
    /*****************************************************************************************************
     * Abstract protected methods - prevents calling these methods outside of the objSQL controller class
     *****************************************************************************************************/
    
    /**
     * Protected methods from extended connection classes
     *
     * @access protected
     */
    protected function obj_db_close() {}
    protected function obj_db_connection() {}
    protected function obj_db_error() {}
    protected function obj_db_escape_data( $str=null ) {}
    protected function obj_db_message() {}
    protected function obj_db_info() {}
	protected function obj_db_rowcount( $table, $cols=false, $where=false ) {}
}

abstract class obj_access 
{
	/**
     * Abstract protected methods from extended connection classes
     *
     * @access protected
     */
    abstract protected function obj_db_close();
    abstract protected function obj_db_connection();
    abstract protected function obj_db_error();
    abstract protected function obj_db_escape_data( $str );
    abstract protected function obj_db_message();
    abstract protected function obj_db_info();
	abstract protected function obj_db_rowcount( $table, $cols, $where );
}

?>